const coreDB = require('qaq-core-db');
const coreUtil = require('qaq-core-util');

exports.getProcessDtls = function(methodParam, connection, callback) {
    var processIdn = methodParam.processIdn || '';
    var coIdn = methodParam.coIdn;
    let processName = methodParam.processName || '';

    let params = [];
    let outJson = {};
    let resultFinal = {};
    let processDtl = {};

    if (processIdn != '' || processName != '') {
        let query = "select nme,in_status,iss_status,out_status,apply_changes,typ,grp,process_idn,rt_status  from stock_process " +
            "where co_idn=$1  and  stt=1 ";
        params.push(coIdn);
        let cnt = 1; 
                
        if (processIdn != ''){
            cnt++;
            query += " and process_idn =$"+cnt;
            params.push(processIdn);
        }
            
        if (processName != ''){
            cnt++;
            query += " and nme =$"+cnt;
            params.push(processName);
        }
            
        // console.log(query);
        // console.log(params);
        coreDB.executeTransSql(connection, query, params, {}, function (error, result) {
            if (error) {
                outJson["result"] = resultFinal;
                outJson["status"] = "FAIL";
                outJson["message"] = "Get Process Details Fail To Execute Query!";
                callback(null, outJson);
            } else {
                let rowCount = result.rowCount;
                if (rowCount > 0) {
                    let processObj = result.rows[0];
                    processDtl["in_status"] = processObj.in_status;
                    processDtl["rt_status"] = processObj.rt_status;
                    processDtl["iss_status"] = processObj.iss_status;
                    processDtl["out_status"] = processObj.out_status;
                    processDtl["apply_changes"] = processObj.apply_changes;
                    processDtl["processType"] = processObj.typ;
                    processDtl["grp"] = processObj.grp;
                    processDtl["process_idn"] = processObj.process_idn;
                    processDtl["processNme"] = processObj.nme;

                    outJson["result"] = processDtl;
                    outJson["status"] = "SUCCESS";
                    outJson["message"] = "SUCCESS";
                    callback(null, outJson);
                } else {
                    outJson["result"] = resultFinal;
                    outJson["status"] = "FAIL";
                    outJson["message"] = "Process Not Available";
                    callback(null, outJson);
                }
            }
        });
    } else if (processIdn == '' || processName == '') {
        outJson["result"] = resultFinal;
        outJson["status"] = "FAIL";
        outJson["message"] = "Please Verify processIdn/processName Parameter";
        callback(null, outJson);
    }
}

exports.insertIssueDataToTemp = function(methodParam, connection, callback) {
    var pktDataList = methodParam.pktDataList || [];
    var coIdn = methodParam.coIdn;
    let pktDataListLen = pktDataList.length;
    let grp = methodParam.grp;
    let params = [];
    let outJson = {};
    let resultFinal = {};

    if (pktDataListLen > 0) {
        let methodParams = {};
        methodParams["tableName"] = "temp_issuereturn_pkts";
        coreUtil.execDropCreateTempTableWithStructure(methodParams, connection).then(execCreateTempTableresult => {
            let stockAttr = "stock_idn";
            let stockType = "bigint";
            if (grp == 'bulkupdate') {
                stockAttr = "pkt_code";
                stockType = "varchar";
            }

            let insertSqlQ = "insert into temp_issuereturn_pkts(" + stockAttr + ", issval, rtnval, empval,stt, flg) " +
                "select *,0 stt,'C' flg from jsonb_to_recordset('" + JSON.stringify(pktDataList).replace(/'/g, "''") + "'::jsonb) as x(id " + stockType + ", issue jsonb,rtn jsonb, emp jsonb) ";

            params = [];
            fmt = {};
            //console.log(insertSqlQ);
            //console.log(params);
            coreDB.executeTransSql(connection, insertSqlQ, params, fmt, async function (error, result) {
                if (error) {
                    outJson["status"] = "FAIL";
                    outJson["message"] = "Fail To Execute Insert into Temp Table Function Query! " + error.message;
                    callback(null, outJson);
                } else {
                    outJson["status"] = "SUCCESS";
                    outJson["message"] = "Temp Table Created Successfully! ";
                    callback(null, outJson);
                }
            })
        })
    } else if (pktDataListLen == 0) {
        outJson["result"] = resultFinal;
        outJson["status"] = "FAIL";
        outJson["message"] = "Please Verify pktDataList Parameter";
        callback(null, outJson);
    }
}

exports.checkTransactionQCM = function(redirectParam, tpoolconn, callback) {
    var processIdn = redirectParam.processIdn || '';
    var empIdn = redirectParam.empIdn || '';
    var coIdn = redirectParam.coIdn;
    let logUsr = redirectParam.logUsr;
    let reissue = redirectParam.reissue || 'N';
    let exhrate = redirectParam.exhrate;
    let subCoIdn = redirectParam.subCoIdn || '';

    let params = [];
    let fmt = {};
    let outJson = {};
    let resultFinal = {};

    if (processIdn != '' && empIdn != '') {
        let sql = "select transaction_qc_idn from transaction_qc  " +
            "where co_idn=$1 and process_idn=$2 and emp_idn=$3   " +
            "and CAST(trns_ts as Date) = current_date ";

        params.push(coIdn);
        params.push(processIdn);
        params.push(empIdn);


        // console.log(sql);
        // console.log(params);
        coreDB.executeTransSql(tpoolconn, sql, params, fmt, async function (error, result) {
            if (error) {
                outJson["status"] = "FAIL";
                outJson["message"] = "Error In checkTransactionQCM Method!" + error.message;
                callback(null, outJson);
            } else {
                var rowCount = result.rowCount;
                if (rowCount > 0 && reissue == 'N') {
                    var resultRow = result.rows[0] || {};
                    outJson["transaction_qc_idn"] = resultRow["transaction_qc_idn"];
                    outJson["status"] = "SUCCESS";
                    outJson["message"] = "SUCCESS";
                    callback(null, outJson);
                } else {
                    let methodParam = {};
                    methodParam["processIdn"] = processIdn;
                    methodParam["empIdn"] = empIdn;
                    methodParam["coIdn"] = coIdn;
                    methodParam["logUsr"] = logUsr;
                    methodParam["status"] = "IS";
                    methodParam["exhrate"] = exhrate;
                    methodParam["subCoIdn"] = subCoIdn;
                    let insertResult = await execInsertTransactionQcM(methodParam, tpoolconn);
                    callback(null, insertResult);
                }
            }
        });
    } else if (processIdn == '') {
        outJson["result"] = resultFinal;
        outJson["status"] = "FAIL";
        outJson["message"] = "Please Verify processIdn Can not be blank!";
        callback(null, outJson);
    } else if (empIdn == '') {
        outJson["result"] = resultFinal;
        outJson["status"] = "FAIL";
        outJson["message"] = "Please Verify empIdn  Can not be blank!";
        callback(null, outJson);
    }
}

function execInsertTransactionQcM(methodParam, tpoolconn) {
    return new Promise(function (resolve, reject) {
        insertTransactionQcM(methodParam, tpoolconn, function (error, result) {
            if (error) {
                reject(error);
            }
            resolve(result);
        });
    });
}

function insertTransactionQcM(methodParam, tpoolconn, callback) {
    var processIdn = methodParam.processIdn || '';
    var empIdn = methodParam.empIdn || '';
    var status = methodParam.status;
    var optionDtl = methodParam.optionDtl || {};
    var coIdn = methodParam.coIdn;
    var logUsr = methodParam.logUsr;
    let exhrate = methodParam.exhrate || '';
    let subCoIdn = methodParam.subCoIdn || '';
    let addl_attr = {};
    if(exhrate != '')
        addl_attr["exhrate"] = exhrate;
    if(subCoIdn != '')
        optionDtl["sub_co_idn"] = subCoIdn;

    let params = [];
    let fmt = {};
    let outJson = {};
    let resultFinal = {};

    if (processIdn != '' && empIdn != '') {
        let insertTransactionQ = "insert into transaction_qc(process_idn,co_idn,co_serial,emp_idn,status,trns_ts,transaction_attr,approved_yn,created_ts,created_by,addl_attr) \n" +
            "values($1,$2,$3,$4,$5,current_timestamp,'" + JSON.stringify(optionDtl) + "',0,current_timestamp,$6,'" + JSON.stringify(addl_attr) + "') RETURNING transaction_qc_idn";

        params.push(processIdn);
        params.push(coIdn);
        params.push("1");
        params.push(empIdn);
        params.push(status);
        params.push(logUsr);
        // console.log(insertTransactionQ);
        // console.log(params);
        coreDB.executeTransSql(tpoolconn, insertTransactionQ, params, fmt, function (error, result) {
            if (error) {
                coreDB.doTransRollBack(tpoolconn);
                outJson["status"] = "FAIL";
                outJson["message"] = "Error In insertTransactionM Method!" + error.message;
                callback(null, outJson);
            } else {
                //coreDB.doTransCommit(tpoolconn);
                var rowCount = result.rowCount;
                if (rowCount > 0) {
                    var resultRow = result.rows[0] || {};
                    outJson["transaction_qc_idn"] = resultRow["transaction_qc_idn"];
                    outJson["status"] = "SUCCESS";
                    outJson["message"] = "TransactionQC Inserted Sucessfully!";
                    callback(null, outJson);
                } else {
                    coreDB.doTransRollBack(tpoolconn);
                    outJson["status"] = "FAIL";
                    outJson["message"] = "TransactionQC Inserted Failed!";
                    callback(null, outJson);
                }
            }
        });
    } else if (processIdn == '') {
        outJson["result"] = resultFinal;
        outJson["status"] = "FAIL";
        outJson["message"] = "Please Verify processIdn Can not be blank!";
        callback(null, outJson);
    } else if (empIdn == '') {
        outJson["result"] = resultFinal;
        outJson["status"] = "FAIL";
        outJson["message"] = "Please Verify empIdn  Can not be blank!";
        callback(null, outJson);
    }
}

exports.validateTempPackets = function(methodParam, tpoolconn, callback) {
    var inStatus = methodParam.inStatus || [];
    var coIdn = methodParam.coIdn;
    let inStatusLen = inStatus.length;
    let grp = methodParam.grp;
    let attr = methodParam.attr || '';
    let transaction_qc_idn = methodParam.transaction_qc_idn;

    let params = [];
    let resultFinal = {};
    let updatedPktIdnList = [];
    let outJson = {};

    if (inStatusLen > 0) {
        let conQ = "";
        if (grp == 'bulkupdate' && attr != '')
            conQ = " , issval = tm.issval || concat('{\"" + attr + "\":\"', regexp_replace(COALESCE(sm.attr->> '" + attr + "',''), '\t', '', 'g') , '\"}')::jsonb ";

        let query = " pkt_code = sm.pkt_code, ";
        let queryLock = "tm.stock_idn=sm.stock_idn";
        if (grp == 'bulkupdate') {
            query = " stock_idn = sm.stock_idn, ";
            queryLock = "tm.pkt_code=sm.pkt_code";
        }


        let sql = "update temp_issuereturn_pkts tm set " + query + " transaction_qc_idn = $1, " +
            "qty = sm.qty_on_hand,weight = cast(sm.attr->> 'crtwt' as real),stock_status=sm.status,stt=1  " + conQ +
            "from stock_m sm where " + queryLock + " and sm.co_idn=$2 " +
            "and sm.stt=1 ";
        if (grp != 'bulkupdate')
            sql += "and sm.status = ANY(ARRAY['" + inStatus.join("','") + "']) ";

        sql += " RETURNING tm.stock_idn ";

        params.push(transaction_qc_idn);
        params.push(coIdn);

        //console.log(sql);
        // console.log(params);
        coreDB.executeTransSql(tpoolconn, sql, params, {}, async function (error, result) {
            if (error) {
                outJson["status"] = "FAIL";
                outJson["message"] = "Error In validateTempPackets Method!" + error.message;
                callback(null, outJson);
            } else {
                var rowCount = result.rowCount;
                if (rowCount > 0) {
                    for (let k = 0; k < rowCount; k++) {
                        var resultRow = result.rows[k];
                        updatedPktIdnList.push(resultRow.stock_idn);
                    }
                    outJson["result"] = updatedPktIdnList;
                    outJson["status"] = "SUCCESS";
                    outJson["message"] = "Temp Packet Validated";
                    callback(null, outJson);
                } else {
                    outJson["result"] = resultFinal;
                    outJson["status"] = "FAIL";
                    outJson["message"] = "Please Verify packets status for selected process!";
                    callback(null, outJson);
                }
            }
        });
    } else if (inStatusLen == 0) {
        outJson["result"] = resultFinal;
        outJson["status"] = "FAIL";
        outJson["message"] = "Please Verify inStatus  Can not be blank!";
        callback(null, outJson);
    }
}

exports.updateStockMIssueStatus = function(methodParam, tpoolconn, callback) {
    var coIdn = methodParam.coIdn;
    let issueStatus = methodParam.issueStatus || '';
    let transaction_qc_idn = methodParam.transaction_qc_idn;
    let logUsr = methodParam.logUsr;

    let params = [];
    let resultFinal = {};
    let outJson = {};
    let attrDtl = {};
    attrDtl["last_trans_id"] = transaction_qc_idn;

    if (issueStatus != '' && issueStatus != 'NA') {
        let sql = "update stock_m sm set status =$1 , attr = attr || '" + JSON.stringify(attrDtl) + "' "+
            ",modified_ts= current_timestamp,modified_by= $4 "+
            "from  temp_issuereturn_pkts tm " +
            "where tm.stock_idn=sm.stock_idn and sm.co_idn=$2 " +
            "and sm.stt=1 and tm.stt=1 and transaction_qc_idn = $3	";

        params.push(issueStatus);
        params.push(coIdn);
        params.push(transaction_qc_idn);
        params.push(logUsr);
        // console.log(sql);
        // console.log(params);
        coreDB.executeTransSql(tpoolconn, sql, params, {}, async function (error, result) {
            if (error) {
                outJson["status"] = "FAIL";
                outJson["message"] = "Error In validateTempPackets Method!" + error.message;
                callback(null, outJson);
            } else {
                var rowCount = result.rowCount;

                outJson["result"] = resultFinal;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "Stock Master updated with issue status";
                callback(null, outJson);
            }
        });
    } else if (issueStatus == '' || issueStatus == 'NA') {
        outJson["result"] = resultFinal;
        outJson["status"] = "SUCCESS";
        outJson["message"] = "Please Verify issueStatus  Can not be blank!";
        callback(null, outJson);
    }
}

exports.insertTransactionQcD = function(methodParam, tpoolconn, callback) {
    var transaction_qc_idn = methodParam.transaction_qc_idn;
    var processType = methodParam.processType;
    var logUsr = methodParam.logUsr;
    let coIdn = methodParam.coIdn;

    var resultFinal = {};
    var outJson = {};
    let QCDStatus = "IS";
    //if (processType != 'IS')
    //    QCDStatus = "RT";
    let insertTransactionQ = "";
    var params = [];

    if(processType != 'IS'){
        insertTransactionQ = "insert into transaction_d_qc(transaction_qc_idn, stock_idn, qty, weight,stock_status,status, issval, rtnval,created_by,rtn_emp_idn,sync_stt,modified_ts,modified_by) " +
        "select $1, stock_idn, qty, weight,stock_status,$2,issval,rtnval,$3,empval,$4,current_timestamp,$5 from  temp_issuereturn_pkts where stt=1 ";

        params.push(transaction_qc_idn);
        params.push(QCDStatus);
        params.push(logUsr);
        params.push("I");
        params.push(logUsr);

    } else {
        insertTransactionQ = "insert into transaction_d_qc(transaction_qc_idn, stock_idn, qty, weight,stock_status,status, issval, rtnval,created_by,rtn_emp_idn,sync_stt) " +
        "select $1, stock_idn, qty, weight,stock_status,$2,issval,rtnval,$3,empval,$4 from  temp_issuereturn_pkts where stt=1 ";

        params.push(transaction_qc_idn);
        params.push(QCDStatus);
        params.push(logUsr);
        params.push("P");
    }
    

    //console.log(insertTransactionQ);
    // console.log(params);
    coreDB.executeTransSql(tpoolconn, insertTransactionQ, params, {}, async function (error, result) {
        if (error) {
            coreDB.doTransRollBack(tpoolconn);
            let redirectParam = {};
            redirectParam["logUsr"] = logUsr;
            redirectParam["coIdn"] = coIdn;
            redirectParam["errorMsg"] = error.message;
            redirectParam["transaction_qc_idn"] = transaction_qc_idn;
            let errorResult = await coreUtil.execUpdateError(redirectParam, tpoolconn);
            //console.log("d_qc error",error)
            outJson["result"] = resultFinal;
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In insertTransactionQcD Method!" + error.message;
            callback(null, outJson);
        } else {
            let rowCount = result.rowCount;
            if (rowCount > 0) {
                //coreDB.doTransCommit(tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "Transaction D Insertion Successfully!";
                callback(null, outJson);
            } else {
                coreDB.doTransRollBack(tpoolconn);
                let redirectParam = {};
                redirectParam["logUsr"] = logUsr;
                redirectParam["coIdn"] = coIdn;
                redirectParam["errorMsg"] = "Transaction D Inserted Failed!";
                redirectParam["transaction_qc_idn"] = transaction_qc_idn;
                let errorResult = await coreUtil.execUpdateError(redirectParam, tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "FAIL";
                outJson["message"] = "Transaction D Inserted Failed!";
                callback(null, outJson);
            }
        }
    });
}

exports.postAssortIssueProcedure = function(methodParam, tpoolconn, callback) {
    var transaction_qc_idn = methodParam.transaction_qc_idn;
    let coIdn = methodParam.coIdn;
    let logUsr = methodParam.logUsr;
    var resultFinal = {};
    var outJson = {};

    let sql = "select post_assort_issue($1, $2) ";

    var params = [];
    params.push(coIdn);
    params.push(transaction_qc_idn);

    //console.log(sql);
    //console.log(params);
    coreDB.executeTransSql(tpoolconn, sql, params, {}, async function (error, result) {
        if (error) {
            coreDB.doTransRollBack(tpoolconn);
            let redirectParam = {};
            redirectParam["logUsr"] = logUsr;
            redirectParam["coIdn"] = coIdn;
            redirectParam["errorMsg"] = "post_assort_issue "+error.message;
            redirectParam["transaction_qc_idn"] = transaction_qc_idn;
            let errorResult = await coreUtil.execUpdateError(redirectParam, tpoolconn);
            console.log(error.message)
            outJson["result"] = resultFinal;
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In post_assort_issue Method!" + error.message;
            callback(null, outJson);
        } else {
            let rowCount = result.rowCount;
            if (rowCount > 0) {
                //coreDB.doTransCommit(tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "post_assort_issue done Successfully!";
                callback(null, outJson);
            } else {
                coreDB.doTransRollBack(tpoolconn);
                let redirectParam = {};
                redirectParam["logUsr"] = logUsr;
                redirectParam["coIdn"] = coIdn;
                redirectParam["errorMsg"] = "post_assort_issue procedure Failed!";
                redirectParam["transaction_qc_idn"] = transaction_qc_idn;
                let errorResult = await coreUtil.execUpdateError(redirectParam, tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "FAIL";
                outJson["message"] = "post_assort_issue procedure Failed!";
                callback(null, outJson);
            }
        }
    });
}

exports.preAssortReturnProcedure = function(methodParam, tpoolconn, callback) {
    var transaction_qc_idn = methodParam.transaction_qc_idn;
    let coIdn = methodParam.coIdn;
    let logUsr = methodParam.logUsr;
    var resultFinal = {};
    var outJson = {};

    let sql = "select pre_assort_return($1, $2) ";


    var params = [];
    params.push(coIdn);
    params.push(transaction_qc_idn);

    //console.log(sql);
    //console.log(params);
    coreDB.executeTransSql(tpoolconn, sql, params, {}, async function (error, result) {
        if (error) {
            coreDB.doTransRollBack(tpoolconn);
            let redirectParam = {};
            redirectParam["logUsr"] = logUsr;
            redirectParam["coIdn"] = coIdn;
            redirectParam["errorMsg"] = "pre_assort_return "+error.message;
            redirectParam["transaction_qc_idn"] = transaction_qc_idn;
            let errorResult = await coreUtil.execUpdateError(redirectParam, tpoolconn);
            console.log(error.message)
            outJson["result"] = resultFinal;
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In pre_assort_return Method!" + error.message;
            callback(null, outJson);
        } else {
            let rowCount = result.rowCount;
            if (rowCount > 0) {
                //coreDB.doTransCommit(tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "pre_assort_return done Successfully!";
                callback(null, outJson);
            } else {
                coreDB.doTransRollBack(tpoolconn);
                let redirectParam = {};
                redirectParam["logUsr"] = logUsr;
                redirectParam["coIdn"] = coIdn;
                redirectParam["errorMsg"] = "pre_assort_return procedure Failed!";
                redirectParam["transaction_qc_idn"] = transaction_qc_idn;
                let errorResult = await coreUtil.execUpdateError(redirectParam, tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "FAIL";
                outJson["message"] = "pre_assort_return procedure Failed!";
                callback(null, outJson);
            }
        }
    });
}

exports.updateStockMIssueAttr = function(methodParam, tpoolconn, callback) {
    var coIdn = methodParam.coIdn;
    let transaction_qc_idn = methodParam.transaction_qc_idn;

    let params = [];
    let resultFinal = {};
    let outJson = {};

    if (transaction_qc_idn != '') {
        let sql = "update stock_m s set attr = attr || issedit "+
            "from trans_pre_issue_attr_v t "+
            "where s.stock_idn = t.stock_idn and t.transaction_qc_idn = $1 and s.co_idn=$2";

        params.push(transaction_qc_idn);
        params.push(coIdn);
        // console.log(sql);
        // console.log(params);
        coreDB.executeTransSql(tpoolconn, sql, params, {}, async function (error, result) {
            if (error) {
                console.log("error",error)
                outJson["status"] = "FAIL";
                outJson["message"] = "Error In update stock_m attr Method!" + error.message;
                callback(null, outJson);
            } else {
                var rowCount = result.rowCount;
                //console.log("rowCount",rowCount);
                outJson["result"] = resultFinal;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "Stock Master updated with issue edit attr";
                callback(null, outJson);
            }
        });
    } else if (transaction_qc_idn == '') {
        outJson["result"] = resultFinal;
        outJson["status"] = "SUCCESS";
        outJson["message"] = "Please Verify transaction_qc_idn  can not be blank!";
        callback(null, outJson);
    }
}

exports.insertStockMGrps = function(methodParam, tpoolconn, callback) {
    var updatedPktIdnList = methodParam.updatedPktIdnList || [];
    var processIdn = methodParam.processIdn;
    var logUsr = methodParam.logUsr;
    let coIdn = methodParam.coIdn;
    let transaction_qc_idn = methodParam.transaction_qc_idn || '';
    let transaction_mix_qc_idn = methodParam.transaction_mix_qc_idn || '';

    var resultFinal = {};
    var outJson = {};

    let insertQ = "insert into stock_m_groups(stock_idn,group_idn,process_idn,attr,stt,created_by) " +
        "select sm.stock_idn,$1,$2,sm.attr,1,$3 from stock_m sm , temp_issuereturn_pkts tm where sm.stock_idn = tm.stock_idn and sm.co_idn=$4 and sm.stt=1 " +
        "and  tm.stt=1  ";

    var params = [];
    params.push("1");
    params.push(processIdn);
    params.push(logUsr);
    params.push(coIdn);


    if (transaction_qc_idn != '') {
        insertQ += " and transaction_qc_idn = $5 ";
        params.push(transaction_qc_idn);
    }

    if (transaction_mix_qc_idn != '') {
        insertQ += " and transaction_qc_idn = $5 ";
        params.push(transaction_mix_qc_idn);
    }

    //console.log(insertQ);
    // console.log(params);
    coreDB.executeTransSql(tpoolconn, insertQ, params, {}, async function (error, result) {
        if (error) {
            coreDB.doTransRollBack(tpoolconn);
            let redirectParam = {};
            redirectParam["logUsr"] = logUsr;
            redirectParam["coIdn"] = coIdn;
            redirectParam["errorMsg"] = error.message;
            if(transaction_mix_qc_idn != ''){
                redirectParam["transaction_mix_qc_idn"] = transaction_mix_qc_idn;
                let errorResult = await coreUtil.execUpdateErrorMix(redirectParam, tpoolconn);    
            } else {
                redirectParam["transaction_qc_idn"] = transaction_qc_idn;
                let errorResult = await coreUtil.execUpdateError(redirectParam, tpoolconn);    
            }
            
            outJson["result"] = resultFinal;
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In insertStockMGrps Method!" + error.message;
            callback(null, outJson);
        } else {
            let rowCount = result.rowCount;
            if (rowCount > 0) {
                //coreDB.doTransCommit(tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "StockM Groups Insertion Successfully!";
                callback(null, outJson);
            } else {
                coreDB.doTransRollBack(tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "FAIL";
                outJson["message"] = "StockM Groups Inserted Failed!";
                callback(null, outJson);
            }
        }
    });
}

exports.updateStockMAttr = function(methodParam, tpoolconn, callback) {
    var logUsr = methodParam.logUsr;
    var coIdn = methodParam.coIdn || '';
    let transaction_qc_idn = methodParam.transaction_qc_idn || '';
    let attr = methodParam.attr || '';
    let transaction_mix_qc_idn = methodParam.transaction_mix_qc_idn || '';

    let params = [];
    let outJson = {};
    let resultFinal = {};
    let conQ = '';
    if(attr == 'rfidcd'){
        conQ = " , tag_id = COALESCE(d.rtnval ->> 'rfidcd','') ";
    }

    if(attr == 'vnm'){
        conQ = " , pkt_code = COALESCE(d.rtnval ->> 'vnm','') ";
    }

    var updateStock = "update stock_m sm set attr = sm.attr || d.rtnval  " + conQ +
        " ,modified_ts=current_timestamp,modified_by=$1 " +
        "from temp_issuereturn_pkts d where sm.stock_idn = d.stock_idn " +
        "and sm.co_idn=$2 " +
        "and sm.stt=1 and d.stt=1 ";

    params = [];
    params.push(logUsr);
    params.push(coIdn);

    if (transaction_qc_idn != '') {
        updateStock += "  and d.transaction_qc_idn = $3 ";
        params.push(transaction_qc_idn);
    }

    if (transaction_mix_qc_idn != '') {
        updateStock += " and d.transaction_qc_idn = $3 ";
        params.push(transaction_mix_qc_idn);
    }

    //console.log(updateStock)
    //console.log(params)
    coreDB.executeTransSql(tpoolconn, updateStock, params, {}, async function (error, result) {
        if (error) {

            coreDB.doTransRollBack(tpoolconn);
            let redirectParam = {};
            redirectParam["logUsr"] = logUsr;
            redirectParam["coIdn"] = coIdn;
            redirectParam["errorMsg"] = error.message;
            if(transaction_mix_qc_idn != ''){
                redirectParam["transaction_mix_qc_idn"] = transaction_mix_qc_idn;
                let errorResult = await coreUtil.execUpdateErrorMix(redirectParam, tpoolconn);    
            } else {
                redirectParam["transaction_qc_idn"] = transaction_qc_idn;
                let errorResult = await coreUtil.execUpdateError(redirectParam, tpoolconn);    
            }
            outJson["status"] = "FAIL";
            outJson["message"] = "Fail To Update stock_m!" + error.message;
            outJson["result"] = resultFinal;
            callback(null, outJson);
        } else {
            //coreDB.doTransCommit(tpoolconn);
            //console.log("StockM Attr Updated");
            outJson["status"] = "SUCCESS";
            outJson["message"] = "SUCCESS";
            outJson["result"] = resultFinal;
            callback(null, outJson);
        }
    })
}

exports.updateStockMStatusDRT = function(methodParam, tpoolconn, callback) {
    var logUsr = methodParam.logUsr;
    var coIdn = methodParam.coIdn || '';
    let out_status = methodParam.out_status || '';
    let transaction_qc_idn = methodParam.transaction_qc_idn || '';
    let transaction_mix_qc_idn = methodParam.transaction_mix_qc_idn || '';

    let params = [];
    let outJson = {};
    let resultFinal = {};

    let sql = "";
    if (out_status != '' && out_status != 'NA')
        sql = " status = '" + out_status + "' , ";

    var updateStock = "update stock_m sm set   " + sql +
        " modified_ts=current_timestamp,modified_by=$1 " +
        "from temp_issuereturn_pkts d where sm.stock_idn = d.stock_idn " +
        "and sm.co_idn=$2 " +
        "and sm.stt=1 and d.stt=1 ";

    params = [];
    params.push(logUsr);
    params.push(coIdn);

    if (transaction_qc_idn != '') {
        updateStock += "  and d.transaction_qc_idn = $3 ";
        params.push(transaction_qc_idn);
    }
    if (transaction_mix_qc_idn != '') {
        updateStock += " and d.transaction_qc_idn = $3 ";
        params.push(transaction_mix_qc_idn);
    }

    //console.log(updateStock)
    //console.log(params)
    coreDB.executeTransSql(tpoolconn, updateStock, params, {}, async function (error, result) {
        if (error) {

            coreDB.doTransRollBack(tpoolconn);
            let redirectParam = {};
            redirectParam["logUsr"] = logUsr;
            redirectParam["coIdn"] = coIdn;
            redirectParam["errorMsg"] = error.message;
            if(transaction_mix_qc_idn != ''){
                redirectParam["transaction_mix_qc_idn"] = transaction_mix_qc_idn;
                let errorResult = await coreUtil.execUpdateErrorMix(redirectParam, tpoolconn);    
            } else {
                redirectParam["transaction_qc_idn"] = transaction_qc_idn;
                let errorResult = await coreUtil.execUpdateError(redirectParam, tpoolconn);    
            }
            outJson["status"] = "FAIL";
            outJson["message"] = "Fail To Update stock_m!" + error.message;
            outJson["result"] = resultFinal;
            callback(null, outJson);
        } else {
            //coreDB.doTransCommit(tpoolconn);
            //console.log("StockM Attr Updated");
            let redirectParam = {};
            redirectParam["logUsr"] = logUsr;
            redirectParam["coIdn"] = coIdn;
            if(transaction_mix_qc_idn != ''){
                redirectParam["transaction_mix_qc_idn"] = transaction_mix_qc_idn;
                let qcResult = await execUpdatetranqcStatusMix(redirectParam, tpoolconn);
            } else {
                redirectParam["transaction_qc_idn"] = transaction_qc_idn;
                let qcResult = await execUpdatetranqcStatus(redirectParam, tpoolconn);
            }
            outJson["status"] = "SUCCESS";
            outJson["message"] = "SUCCESS";
            outJson["result"] = resultFinal;
            callback(null, outJson);
        }
    })
}

function execUpdatetranqcStatus(methodParam, tpoolconn) {
    return new Promise(function (resolve, reject) {
        UpdatetranqcStatus(methodParam, tpoolconn, function (error, result) {
            if (error) {
                reject(error);
            }
            resolve(result);
        });
    })
}

function UpdatetranqcStatus(methodParam, tpoolconn, callback) {
    let outJson = {};
    let coIdn = methodParam.coIdn;
    let transaction_qc_idn = methodParam.transaction_qc_idn;
    let logUsr = methodParam.logUsr;
    let params = [];

    let updateQC = "update transaction_d_qc  a set status='RT' ,modified_ts=current_timestamp,modified_by=$1,sync_stt='P'  from temp_issuereturn_pkts b " +
        " where a.transaction_qc_idn=b.transaction_qc_idn and a.stock_idn=b.stock_idn ";

    params.push(logUsr);
    //console.log(transaction_qc_idn);
    if (transaction_qc_idn != '') {
        updateQC += "  and a.transaction_qc_idn = $2 ";
        params.push(transaction_qc_idn);
    }

    coreDB.executeTransSql(tpoolconn, updateQC, params, {}, function (error, result) {
        if (error) {
            console.log(error);
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In transaction_d_qc updation !" + error.message;
            callback(null, outJson);
        } else {
            //console.log("result",result.rowCount)
            outJson["status"] = "SUCCESS";
            outJson["message"] = "SUCCESS";
            callback(null, outJson);
        }

    });

}

function execUpdatetranqcStatusMix(methodParam, tpoolconn) {
    return new Promise(function (resolve, reject) {
        UpdatetranqcStatusMix(methodParam, tpoolconn, function (error, result) {
            if (error) {
                reject(error);
            }
            resolve(result);
        });
    })
}

function UpdatetranqcStatusMix(methodParam, tpoolconn, callback) {
    let outJson = {};
    let coIdn = methodParam.coIdn;
    let transaction_mix_qc_idn = methodParam.transaction_mix_qc_idn;
    let logUsr = methodParam.logUsr;
    let params = [];

    let updateQC = "update transaction_mix_d_qc  a set status='RT' ,modified_ts=current_timestamp,modified_by=$1,sync_stt='C'  from temp_issuereturn_pkts b " +
        " where a.transaction_mix_qc_idn=b.transaction_qc_idn and a.stock_idn=b.stock_idn ";

    params.push(logUsr);
    //console.log(transaction_qc_idn);
    if (transaction_mix_qc_idn != '') {
        updateQC += "  and a.transaction_mix_qc_idn = $2 ";
        params.push(transaction_mix_qc_idn);
    }

    coreDB.executeTransSql(tpoolconn, updateQC, params, {}, function (error, result) {
        if (error) {
            console.log(error);
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In transaction_d_qc updation !" + error.message;
            callback(null, outJson);
        } else {
            //console.log("result",result.rowCount)
            outJson["status"] = "SUCCESS";
            outJson["message"] = "SUCCESS";
            callback(null, outJson);
        }

    });

}

exports.getFailPktList = function(methodParam, connection, callback) {
    var coIdn = methodParam.coIdn;
    let transaction_qc_idn = methodParam.transaction_qc_idn || '';

    let params = [];
    let outJson = {};
    let invalidPktList = [];
    let errorPktList = [];
    let resultFinal = {};

    params.push(coIdn);
    let query = "select sm.pkt_Code, " +
        "case when tm.stt = 0 then 'Fail' " +
        "when  tm.flg = 'E' then 'Error' end flg  from stock_m sm,temp_issuereturn_pkts tm where " +
        "tm.stock_idn=sm.stock_idn and sm.co_idn=$1 " +
        "and (sm.stt=1 or tm.stt=0) ";

    if (transaction_qc_idn != '') {
        query += " and tm.transaction_qc_idn = $2 ";
        params.push(transaction_qc_idn);
    }
    // console.log(query);
    // console.log(params);
    coreDB.executeTransSql(connection, query, params, {}, function (error, result) {
        if (error) {
            outJson["result"] = failPktList;
            outJson["status"] = "FAIL";
            outJson["message"] = "Get Fail Packet List Fail To Execute Query!";
            callback(null, outJson);
        } else {
            let rowCount = result.rowCount;
            if (rowCount > 0) {
                for (let m = 0; m < rowCount; m++) {
                    let pktObj = result.rows[m];
                    let flg = pktObj.flg;
                    if (flg == 'Error')
                        errorPktList.push(pktObj.pkt_code);
                    if (flg == 'Fail')
                        invalidPktList.push(pktObj.pkt_code);
                }
                if (errorPktList.length > 0)
                    resultFinal["errorPktList"] = errorPktList;
                if (errorPktList.length > 0)
                    resultFinal["invalidPktList"] = invalidPktList;

                outJson["result"] = resultFinal;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "SUCCESS";
                callback(null, outJson);
            } else {
                outJson["result"] = failPktList;
                outJson["status"] = "FAIL";
                outJson["message"] = "Fail Packet List Not Available";
                callback(null, outJson);
            }
        }
    })
}

exports.sendBulkUpdateMail = function(methodParam, callback) {
    //console.log("local",methodParam)
    let poolName = methodParam.poolName;
    let transaction_qc_idn = methodParam.transactionQCIdn || '';
    let transaction_mix_qc_idn = methodParam.transaction_mix_qc_idn || '';
    let userIdn = methodParam.userIdn;
    let coIdn = methodParam.coIdn;
    let pvtConfigure = methodParam.pvtConfigure;
    let source = methodParam.source;
    let logUsr = methodParam.logUsr;
    let prefix = methodParam.prefix || '';
    let outJson = {};

    var poolsList = require('qaq-core-db').poolsList;
    var pool = poolsList[poolName] || '';
    if (pool != '') {
        coreDB.getTransPoolConnect(pool, async function (error, tpoolconn) {
            if (error) {
                outJson["result"] = '';
                outJson["status"] = "FAIL";
                outJson["message"] = "Fail To Get Conection!";
                callback(null, outJson);
            } else {
                //console.log("mail Sending")
                let methodParamlocal = {};
                methodParamlocal["transactionQCIdn"] = transaction_qc_idn;
                methodParamlocal["transaction_mix_qc_idn"] = transaction_mix_qc_idn;
                methodParamlocal["userIdn"] = userIdn;
                methodParamlocal["coIdn"] = coIdn;
                methodParamlocal["pvtConfigure"] = pvtConfigure;
                methodParamlocal["source"] = source;
                methodParamlocal["logUsr"] = logUsr;
                methodParamlocal["prefix"] = prefix;
                let result = await coreUtil.sendAttrUpdateMail(methodParamlocal, tpoolconn);
                coreDB.doTransRelease(tpoolconn);
                //console.log("new connection realese");
                outJson["result"] = '';
                outJson["status"] = "SUCCESS";
                outJson["message"] = "SUCCESS";
                callback(null, outJson);
            }
        })
    } else {
        outJson["result"] = '';
        outJson["status"] = "FAIL";
        outJson["message"] = "Fail To Get Conection!";
        callback(null, outJson);
    }
}

exports.getTransactionSummary = function(methodParam, tpoolconn, callback) {
    let transaction_qc_idn = methodParam.transaction_qc_idn;
    let coIdn = methodParam.coIdn;
    let summaryTrnsDtl = {};
    let outJson = {};

    let sql = "select tq.transaction_qc_idn,tq.co_serial,sp.print,sp.nme, sp.grp,sp.process_idn, "+
            "get_nme(tq.emp_idn) employee,tq.emp_idn, "+
            "sum(tdq.qty) qty,sum(tdq.weight) crtwt "+
            "from transaction_qc tq,transaction_d_qc tdq, "+
            "stock_m sm,stock_process sp "+
            "where tq.transaction_qc_idn = tdq.transaction_qc_idn "+
            "and tq.process_idn= sp.process_idn "+
            "and tdq.stock_idn=sm.stock_idn "+
            "and sm.co_idn=$1 and sp.co_idn=$2 "+
            "and sp.stt=1 and tdq.status='IS' "+
            "and sp.iss_status=sm.status "+
            "and tq.transaction_qc_idn = $3  and sp.co_idn = tq.co_idn"+
            "group by tq.transaction_qc_idn,sp.print,tq.emp_idn, sp.grp,sp.nme,sp.process_idn,tq.co_serial ";


    let params = [];
    params.push(coIdn);
    params.push(coIdn);
    params.push(transaction_qc_idn);

    //console.log(sql);
    // console.log(params);
    coreDB.executeTransSql(tpoolconn, sql, params, {}, async function (error, result) {
        if (error) {
            outJson["result"] = summaryTrnsDtl;
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In getTransactionSummary Method!" + error.message;
            callback(null, outJson);
        } else {
            let rowCount = result.rowCount;
            if (rowCount > 0) {
                summaryTrnsDtl["transactionIdn"] = result.rows[0].transaction_qc_idn;
                summaryTrnsDtl["co_serial"] = result.rows[0].co_serial;
                summaryTrnsDtl["processNme"] = result.rows[0].nme;
                summaryTrnsDtl["qty"] = result.rows[0].qty;
                summaryTrnsDtl["weight"] = result.rows[0].crtwt;
                summaryTrnsDtl["employee"] = result.rows[0].employee;
                summaryTrnsDtl["emp_idn"] = result.rows[0].emp_idn;
                summaryTrnsDtl["processGrp"] = result.rows[0].grp;
                summaryTrnsDtl["processPrint"] = result.rows[0].print;
                summaryTrnsDtl["processIdn"] = result.rows[0].process_idn;

                outJson["result"] = summaryTrnsDtl;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "SUCCESS";
                callback(null, outJson);
            } else {
                outJson["result"] = summaryTrnsDtl;
                outJson["status"] = "FAIL";
                outJson["message"] = "Sorry no result found";
                callback(null, outJson);
            }
        }
    })
}

exports.updateError = function(methodParam, tpoolconn, callback) {
    var logUsr = methodParam.logUsr;
    var coIdn = methodParam.coIdn || '';
    let errorMsg = methodParam.errorMsg || '';
    let transaction_qc_idn = methodParam.transaction_qc_idn || '';
    let statusUpdateYN = methodParam.statusUpdateYN || 'Y';
    let qcStatus = methodParam.qcStatus || '';

    let params = [];
    let outJson = {};
    let resultFinal = {};
    let conQ = '';
    if(statusUpdateYN == 'Y')
        conQ = " status='FL' , ";
    if(qcStatus != '')
        conQ = " status = '"+qcStatus+"' , ";

    var updateQ = "update transaction_d_qc d set "+conQ+" remarks = $1 " +
        ",modified_ts=current_timestamp,modified_by=$2 from temp_issuereturn_pkts tm " +
        "where d.stock_idn=tm.stock_idn  and tm.stt = 1  ";

    params = [];
    params.push(errorMsg);
    params.push(logUsr);

    if (transaction_qc_idn != '') {
        updateQ += " and d.transaction_qc_idn=$3 ";
        params.push(transaction_qc_idn);
    }

    //console.log(updateQ)
    //console.log(params)
    coreDB.executeTransSql(tpoolconn, updateQ, params, {}, function (error, result) {
        if (error) {
            coreDB.doTransRollBack(tpoolconn);
            console.log(error.message)
            outJson["status"] = "FAIL";
            outJson["message"] = "Fail To Update transaction_d_qc!" + error.message;
            outJson["result"] = resultFinal;
            callback(null, outJson);
        } else {
            //console.log("In stock_m");
            var updateStock = "update stock_m sm set status = d.stock_status  " +
                " ,modified_ts=current_timestamp,modified_by=$1 " +
                "from temp_issuereturn_pkts d where sm.stock_idn = d.stock_idn " +
                "and sm.co_idn=$2 " +
                "and sm.stt=1 and d.stt=1 ";

            params = [];
            params.push(logUsr);
            params.push(coIdn);

            if (transaction_qc_idn != '') {
                updateStock += " and transaction_qc_idn=$3 ";
                params.push(transaction_qc_idn);
            }

            //console.log(updateStock)
            //console.log(params)
            coreDB.executeTransSql(tpoolconn, updateStock, params, {}, function (error, result) {
                if (error) {
                    coreDB.doTransRollBack(tpoolconn);
                    outJson["status"] = "FAIL";
                    outJson["message"] = "Fail To Update Stock M!" + error.message;
                    outJson["result"] = resultFinal;
                    callback(null, outJson);
                } else {
                    //console.log("StockM Status Updated");

                    updateQ = "update temp_issuereturn_pkts d set flg='E' ,remarks = $1 " +
                        "where tm.stt = 1 ";

                    params = [];
                    params.push(errorMsg);
                    if (transaction_qc_idn != '') {
                        updateQ += " and transaction_qc_idn=$2 ";
                        params.push(transaction_qc_idn);
                    }

                    //console.log(updateQ)
                    //console.log(params)
                    coreDB.executeTransSql(tpoolconn, updateQ, params, {}, function (error, result) {
                        if (error) {
                            coreDB.doTransRollBack(tpoolconn);
                            outJson["status"] = "FAIL";
                            outJson["message"] = "Fail To Update temp_issuereturn_pkts!" + error.message;
                            outJson["result"] = resultFinal;
                            callback(null, outJson);
                        } else {
                            //coreDB.doTransCommit(tpoolconn);

                            outJson["status"] = "SUCCESS";
                            outJson["message"] = "SUCCESS";
                            outJson["result"] = resultFinal;
                            callback(null, outJson);
                        }
                    })
                }
            })
        }
    })
}

exports.getProcessSettingAttr = function(methodParam, connection, callback) {
    var coIdn = methodParam.coIdn;
    var attr = methodParam.attr;
    let params = [];
    let outJson = {};
    let resultFinal = {};

    let sql = "SELECT config ->> 'val' val,nme  FROM stock_process "+
               " CROSS JOIN jsonb_array_elements(settings -> 'settingsDtl') config "+
              " where 1=1  and  config ->> 'flg'='1' and co_idn=$1 and  config ->> 'attr'=$2 "+
              " and grp  like 'mix_%'";

        params.push(coIdn);
        params.push(attr);
        coreDB.executeTransSql(connection, sql, params, {}, async function (error, result) {
            if (error) {
                outJson["status"] = "FAIL";
                outJson["message"] = "Error In select attr of process Method!" + error.message;
                callback(null, outJson);
            } else {
                var len = result.rows.length;
                if (len > 0) {
                    for (let i = 0; i < len; i++) {
                    var nme = result.rows[i].nme;
                    var val = result.rows[i].val;
                    resultFinal[nme] = val;
                    }
                }
                outJson["status"] = "SUCCESS";
                outJson["message"] = "SUCCESS";
                outJson["result"] = resultFinal;
                callback(null, outJson);
            }

        });


}

exports.getProcessAttr = function(methodParam, connection, callback) {
    var processIdn = methodParam.processIdn || '';
    var coIdn = methodParam.coIdn;
    let productAttributeM = methodParam.productAttributeM;
    let productAttributeDtl = methodParam.productAttributeDtl;
    var processNme = methodParam.processNme || '';
    let returnSatatus = methodParam.returnSatatus || '';

    let params = [];
    let outJson = {};
    let resultFinal = {};

    if (processIdn != '' || processNme != '' || returnSatatus != '') {
        let sql = "select  sp.process_idn,sp.in_status,sp.out_status,sp.rt_status,sp.iss_status,attrDtl ->>'flg' flg,attrDtl ->>'srt' srt, " +
            " attrDtl ->>'attr' processattr,attrDtl ->>'compare' compare,sp.typ,sp.grp,sp.description,sp.nme " +
            "from stock_process sp " +
            "CROSS JOIN jsonb_array_elements(sp.process_attr -> 'process_attrDtl') attrDtl " +
            "where 1=1 " +
            "and sp.co_idn=$1 and sp.stt=1 and attrDtl ->>'stt' = '1' ";

        params.push(coIdn);
        let cnt = 1;
        if(processIdn != ''){
            cnt++;
            sql+=" and sp.process_idn = $"+cnt;
            params.push(processIdn);
        }
        
        if(processNme != ''){
            cnt++;
            sql+=" and sp.nme = $"+cnt;
            params.push(processNme);
        }

        if(returnSatatus != ''){
            cnt++;
            sql+=" and $"+cnt+" = ANY(in_status) ";
            params.push(returnSatatus);
        }


        //console.log(sql);
        //console.log(params);
        coreDB.executeTransSql(connection, sql, params, {}, async function (error, result) {
            if (error) {
                console.log(error);
                outJson["status"] = "FAIL";
                outJson["message"] = "Error In select attr of process Method!" + error.message;
                callback(null, outJson);
            } else {
                var len = result.rows.length;
                if (len > 0) {
                    let processList = [];
                    let processAttrMap = {};
                    let processAttrList = [];
                    let ktsList = [];
                    let ktsAttrList = [];
                    resultFinal["in_status"] = result.rows[0].in_status;
                    resultFinal["out_status"] = result.rows[0].out_status;
                    resultFinal["rt_status"] = result.rows[0].rt_status;
                    resultFinal["processType"] = result.rows[0].typ;
                    resultFinal["iss_status"] = result.rows[0].iss_status;
                    resultFinal["process_idn"] = result.rows[0].process_idn;
                    resultFinal["grp"] = result.rows[0].grp;
                    resultFinal["processName"] = result.rows[0].description;
                    resultFinal["name"] = result.rows[0].nme;
                    for (let i = 0; i < len; i++) {
                        let map = {};
                        let processAttr = result.rows[i].processattr || "";
                        if (processAttr == 'ktsview') {
                            let methodParams = {};
                            methodParams["productAttributeM"] = productAttributeM;
                            methodParams["productAttributeDtl"] = productAttributeDtl;
                            methodParams["coIdn"] = coIdn;
                            let ktsAttrResult = await coreUtil.execGetKtsAttr(methodParams, connection);
                            if (ktsAttrResult.status == 'SUCCESS') {
                                ktsList = ktsAttrResult["result"] || [];
                                ktsAttrList = ktsAttrResult["ktsAttrList"] || [];
                            }


                        }
                        let data_typ = productAttributeM[processAttr + "#T"] || '';
                        map["attr"] = processAttr;
                        let flg = result.rows[i].flg || "";
                        map["flag"] = flg;
                        let srt = result.rows[i].srt || 0;
                        map["sort"] = parseInt(srt);
                        map["compare"] = result.rows[i].compare || "no";
                        let print = productAttributeM[processAttr] || '';
                        map["data_typ"] = data_typ;
                        map["print"] = print;

                        if (data_typ == 'c') {
                            var prpVal = productAttributeDtl[processAttr + "#V"] || [];
                            var prpSort = productAttributeDtl[processAttr + "#S"] || [];
                            map["prpval"] = prpVal;
                            map["prpsort"] = prpSort;
                            var prpPrint = productAttributeDtl[processAttr + "#P"] || [];
                            var prpDisplay = productAttributeDtl[processAttr + "#DI"] || [];
                            map["prpPrint"] = prpPrint;
                            map["prpDisplay"] = prpDisplay;
                        }
                        processAttrMap[processAttr] = flg;
                        processAttrList.push(processAttr);
                        processList.push(map);
                    }
                    processList.sort((val1, val2) => {
                        return (val1.sort) - (val2.sort);
                    });

                    resultFinal["processList"] = processList;
                    resultFinal["processAttrList"] = processAttrList;
                    resultFinal["processAttrMap"] = processAttrMap;
                    resultFinal["ktsList"] = ktsList;
                    resultFinal["ktsAttrList"] = ktsAttrList;

                    outJson["status"] = "SUCCESS";
                    outJson["message"] = "SUCCESS";
                    outJson["result"] = resultFinal;
                    callback(null, outJson);
                } else {
                    outJson["status"] = "FAIL";
                    outJson["message"] = "Please verify selected process";
                    outJson["result"] = resultFinal;
                    callback(null, outJson);
                }
            }
        })
    } else if (processIdn == '' || processNme == '') {
        outJson["result"] = resultFinal;
        outJson["status"] = "FAIL";
        outJson["message"] = "Please Verify processIdn/processNme Can not be blank!";
        callback(null, outJson);
    } else if (returnSatatus == '') {
        outJson["result"] = resultFinal;
        outJson["status"] = "FAIL";
        outJson["message"] = "Please Verify returnSatatus Can not be blank!";
        callback(null, outJson);
    }
}

exports.getStockAttr = function(methodParam, connection, callback) {
    var pktIdnList = methodParam.pktIdnList || [];
    var stockIdnList = methodParam.stockIdnList || [];
    let control_noList = methodParam.control_noList || [];
    var coIdn = methodParam.coIdn;
    let pktIdnListLen = pktIdnList.length;
    var instatusList = methodParam.instatusList || [];
    let resultView = methodParam.resultView;
    let resultViewlen = resultView.length;
    let processAttrMap = methodParam.processAttrMap || {};
    let productAttributeM = methodParam.productAttributeM;
    let productAttributeDtl = methodParam.productAttributeDtl;
    let ktsAttrList = methodParam.ktsAttrList || [];
    let ktsAttrListLen = ktsAttrList.length;
    let where_query_str = methodParam.where_query_str || '';
    let lab_loc = methodParam.lab_loc || 'cb';
    let pktTyp = methodParam.pktTyp || 'NR';
    let processName = methodParam.processName || '';
    //console.log("lab_loc",lab_loc);
    let pktTypList =  pktTyp.split("@");

    let params = [];
    let outJson = {};
    let resultFinal = {};
    let dataTypeFormatDtl = {};
    let formulaAttr = [];
    for (let j = 0; j < resultViewlen; j++) {
        let attr = resultView[j];
        let data_typ = productAttributeM[attr + "#T"] || '';
        let format = productAttributeM[attr + "#F"] || '';
        let formula = productAttributeM[attr + "#FL"] || '';
        dataTypeFormatDtl[attr] = data_typ;
        dataTypeFormatDtl[attr + "#F"] = format;
        if(attr == 'crtwt')
            dataTypeFormatDtl[attr + "#F"] = '';

        dataTypeFormatDtl[attr + "#FL"] = formula;
        if (formula != '') {
            formulaAttr.push(attr);
            dataTypeFormatDtl[attr + "#FL#ATTR"] = coreUtil.getDependAttrList(formula);
        }
    }
    let formulaAttrLength = formulaAttr.length;
    let serviceValList = productAttributeDtl["service#V"] || [];
    let serviceSrtList = productAttributeDtl["service#S"] || [];
     let ddService = serviceSrtList[serviceValList.indexOf("dd")];
      let dgService = serviceSrtList[serviceValList.indexOf("dg")];
      let lab_locValList = productAttributeDtl["lab_loc#V"] || [];
      let lab_locSrtList = productAttributeDtl["lab_loc#S"] || [];
      let cbLoc= lab_locSrtList[lab_locValList.indexOf(lab_loc)];  

     
    let sql = "select sm.stock_idn,sm.pkt_code,sm.stock_type , trunc(Cast(attr ->> 'crtwt' as numeric), 3) wt,sm.status stt,Cast(COALESCE(attr ->> 'qty', '1') as numeric) qtys  ";
    for (let i = 0; i < resultViewlen; i++) {
        let attr = resultView[i];
        if(attr == 'crtwt')
            sql += ",  trunc(Cast(attr ->> 'crtwt' as numeric), 3) crtwt ";
        else
            sql += ", attr ->> '" + attr + "' " + attr;
    }
    for (let i = 0; i < ktsAttrListLen; i++) {
        let attr = ktsAttrList[i];
        sql += ", attr ->> '" + attr + "' " + attr;
    }
    sql += " from stock_m sm " +
        "where 1=1 and sm.co_idn = $1 and Cast(attr ->> 'crtwt' as numeric) > 0 "+
         " and sm.stock_type in ('" + pktTypList.join("','") + "')  and  sm.stt=1 and sm.status = ANY(ARRAY['" + instatusList.join("','") + "']) ";

    params.push(coIdn);
    if (pktIdnListLen > 0) {
        sql += " and sm.pkt_code = ANY(ARRAY['" + pktIdnList.join("','") + "']) ";
    }
    if(stockIdnList.length>0){
        sql +=  " and sm.stock_idn = ANY(ARRAY[" + stockIdnList.join(",") + "]) ";
    }
    if(control_noList.length>0){
        sql +=  " and sm.attr ->> 'control_no' = ANY(ARRAY['" + control_noList.join("','") + "']) ";
    }
    if (where_query_str != '')
        sql += " and " + where_query_str

    sql += " order by sm.sort ";


    //console.log(sql);
    //console.log(params);
    coreDB.executeTransSql(connection, sql, params, {}, async function (error, result) {
        if (error) {
            console.log(error);
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In select attr of packets Method!" + error.message;
            callback(null, outJson);
        } else {
            var len = result.rows.length;
            if (len > 0) {
                let packetIssue = {};
                let packetDisplay = [];
                let packetReturn = {};
                let packetKtsIssue = {};
                let inputList = [];
                for (let i = 0; i < len; i++) {
                    let data = result.rows[i];
                    let issueMap = {};
                    let inputMap = {};
                    let disMap = {};
                    let returnMap = {};
                    let ktsMap = {};
                    let pkt_code = data.pkt_code || "";
                    let stock_idn = data.stock_idn || "";
                    let stock_type = data.stock_type;
                    let crtwt = data.wt;
                    disMap["pkt_code"] = pkt_code;
                    disMap["stock_idn"] = stock_idn;
                    inputMap["id"] = stock_idn;
                    for (let j = 0; j < resultViewlen; j++) {
                        let attr = resultView[j];
                        let data_typ = dataTypeFormatDtl[attr] || '';
                        let attrVal = data[attr] || '';
                        issueMap[attr] = attrVal;
                        let flag = processAttrMap[attr];
                        if (data_typ == 'c' && attrVal != '' && attrVal != 0) {
                            if (flag != 'comp')
                                returnMap[attr] = attrVal;
                            else
                                returnMap[attr] = "";
                          

                            disMap[attr+"_sort"] = attrVal;
                            let prpSort = productAttributeDtl[attr + "#S"] || [];
                            let prpDisplay = productAttributeDtl[attr + "#P"] || [];
                            attrVal = prpDisplay[prpSort.indexOf(parseInt(attrVal))];
                        } else if (data_typ == 'd' && attrVal != '' && attrVal != 0) {
                            //console.log("stock_idn",stock_idn);
                            //console.log(attr+"attrVal"+attrVal);
                            attrVal = coreUtil.getDateyyyymmddToExpected(attrVal, 'dd-mmm-yyyy');
                            if (flag != 'comp')
                                returnMap[attr] = attrVal;
                            else
                                returnMap[attr] = "";
                        } else if (data_typ == 'n' && attrVal != '' && attrVal != 0) {
                            let format = dataTypeFormatDtl[attr + "#F"] || 'N';
                            if (format != 'N') {
                                if (stock_type != 'NR' && attr == "crtwt") {
                                    format = 3;
                                }
                                attrVal = attrVal || 0;
                                attrVal = coreUtil.floorFigure(attrVal, format);
                            }                    
                                  if (flag != 'comp')
                                returnMap[attr] = attrVal;
                            else
                                returnMap[attr] = "";
                        } else {
                            if (flag != 'comp')
                                returnMap[attr] = attrVal;
                            else
                                returnMap[attr] = "";
                        }
                        
                        if(attr=='service'){
                            let serviceVal = issueMap[attr] || '';
                            //if(processName == 'lab_issue')
                            //    serviceVal = '';               
                            if(serviceVal == '' || serviceVal == '10'){
                                if(flag=='issedit'  && crtwt>=1){
                                    issueMap[attr] = dgService;
                                    returnMap[attr] = dgService;
                                }else  if(flag=='issedit' && crtwt<1){
                                    issueMap[attr] = ddService;
                                    returnMap[attr] = ddService;
                                }
                            }
                        }
                        
                        if(flag=='issedit' && attr=='lab_loc'){
                            issueMap[attr] = cbLoc;
                            returnMap[attr] = cbLoc;
                        }

                        disMap[attr] = attrVal;
                    }
                    disMap["status"] = data["stt"];

                    for (let f = 0; f < formulaAttrLength; f++) {
                        let attr = formulaAttr[f];
                        let formula = dataTypeFormatDtl[attr + "#FL"] || '';
                        if (formula != '') {
                            let dependAttr = dataTypeFormatDtl[attr + "#FL#ATTR"] || [];
                            let dependAttrLength = dependAttr.length;
                            if (dependAttrLength > 0) {
                                let attrVal = 'N';
                                for (let l = 0; l < dependAttrLength; l++) {
                                    let depAttr = dependAttr[l];
                                    if (resultView.indexOf(depAttr) > -1) {
                                        formula = replaceall("#" + depAttr, disMap[depAttr] || "0", formula);
                                        attrVal = 'Y';
                                    } else {
                                        attrVal = 'N'
                                        break;
                                    }
                                }
                                if (attrVal != 'N') {
                                    attrVal = eval(formula);
                                    let format = dataTypeFormatDtl[attr + "#F"] || 'N';
                                    if (format != 'N') {
                                        attrVal = coreUtil.floorFigure(attrVal, format);
                                    }
                                    disMap[attr] = attrVal;
                                } else
                                    disMap[attr] = '';
                            } else {
                                disMap[attr] = '';
                            }
                        } else {
                            disMap[attr] = '';
                        }
                    }
                    disMap["qty"] = data["qtys"];
                    issueMap["qty"] = data["qtys"];
                    returnMap["qty"] = data["qtys"];
                    for (let k = 0; k < ktsAttrListLen; k++) {
                        let attr = ktsAttrList[k];
                        let attrVal = data[attr] || '';
                        if (attrVal != '')
                            ktsMap[attr] = attrVal;
                    }
                    packetIssue[stock_idn] = issueMap;
                    inputMap["issue"] = issueMap;
                    packetDisplay.push(disMap);
                    inputMap["rtn"] = returnMap;
                    packetReturn[stock_idn] = returnMap;
                    packetKtsIssue[stock_idn] = ktsMap;
                    inputMap["emp"] = {};
                    inputList.push(inputMap);
                }

                resultFinal["packetDisplay"] = packetDisplay;
                resultFinal["packetIssue"] = packetIssue;
                resultFinal["packetReturn"] = packetReturn;
                resultFinal["packetKtsIssue"] = packetKtsIssue;
                resultFinal["pktDataList"] = inputList;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "SUCCESS";
                outJson["result"] = resultFinal;
                callback(null, outJson);
            } else {
                outJson["status"] = "FAIL";
                outJson["message"] = "Packet not found for selected process";
                outJson["result"] = resultFinal;
                callback(null, outJson);
            }
        }
    })
}

exports.postAssortReturnProc = function(redirectParam, tpoolconn, callback) {
    var transaction_qc_idn = redirectParam.transaction_qc_idn;
    let coIdn = redirectParam.coIdn;
    let stock_idn = redirectParam.stock_idn;
    let processIdn = redirectParam.processIdn;
    let logUsr = redirectParam.logUsr;
    var resultFinal = {};
    var outJson = {};

    let sql = "select post_assort_return($1, $2, $3, $4)";


    var params = [];
    params.push(coIdn);
    params.push(processIdn);
    params.push(transaction_qc_idn);
    params.push(stock_idn);

    //console.log(sql);
    //console.log(params);
    coreDB.executeTransSql(tpoolconn, sql, params, {}, async function (error, result) {
        if (error) {
            coreDB.doTransRollBack(tpoolconn);
            let redirectParam = {};
            redirectParam["logUsr"] = logUsr;
            redirectParam["coIdn"] = coIdn;
            redirectParam["errorMsg"] = "post_assort_return "+error.message;
            redirectParam["transaction_qc_idn"] = transaction_qc_idn;
            redirectParam["statusUpdateYN"] = "N";
            redirectParam["qcStatus"] = "IS";
            let errorResult = await coreUtil.execUpdateError(redirectParam, tpoolconn);
            console.log(error.message)
            outJson["result"] = resultFinal;
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In post_assort_return Method!" + error.message;
            callback(null, outJson);
        } else {
            let rowCount = result.rowCount;
            if (rowCount > 0) {
                //coreDB.doTransCommit(tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "post_assort_return done Successfully!";
                callback(null, outJson);
            } else {
                coreDB.doTransRollBack(tpoolconn);
                let redirectParam = {};
                redirectParam["logUsr"] = logUsr;
                redirectParam["coIdn"] = coIdn;
                redirectParam["errorMsg"] = "post_assort_return procedure Failed!";
                redirectParam["transaction_qc_idn"] = transaction_qc_idn;
                redirectParam["statusUpdateYN"] = "N";
                redirectParam["qcStatus"] = "IS";
                let errorResult = await coreUtil.execUpdateError(redirectParam, tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "FAIL";
                outJson["message"] = "post_assort_return procedure Failed!";
                callback(null, outJson);
            }
        }
    });
}

exports.checkTransactionMixQCM = function(redirectParam, tpoolconn, callback) {
    var processIdn = redirectParam.processIdn || '';
    var empIdn = redirectParam.empIdn || '';
    var coIdn = redirectParam.coIdn;
    let logUsr = redirectParam.logUsr;
    let reissue = redirectParam.reissue || 'N';
    let exhrate = redirectParam.exhrate;

    let params = [];
    let fmt = {};
    let outJson = {};
    let resultFinal = {};

    if (processIdn != '' && empIdn != '') {
        let sql = "select transaction_mix_qc_idn from transaction_mix_qc  " +
            "where co_idn=$1 and process_idn=$2 and emp_idn=$3   " +
            "and CAST(trns_ts as Date) = current_date ";

        params.push(coIdn);
        params.push(processIdn);
        params.push(empIdn);


        // console.log(sql);
        // console.log(params);
        coreDB.executeTransSql(tpoolconn, sql, params, fmt, async function (error, result) {
            if (error) {
                outJson["status"] = "FAIL";
                outJson["message"] = "Error In checkTransactionMixQCM Method!" + error.message;
                callback(null, outJson);
            } else {
                var rowCount = result.rowCount;
                if (rowCount > 0 && reissue == 'N') {
                    var resultRow = result.rows[0] || {};
                    outJson["transaction_mix_qc_idn"] = resultRow["transaction_mix_qc_idn"];
                    outJson["status"] = "SUCCESS";
                    outJson["message"] = "SUCCESS";
                    callback(null, outJson);
                } else {
                    let methodParam = {};
                    methodParam["processIdn"] = processIdn;
                    methodParam["empIdn"] = empIdn;
                    methodParam["coIdn"] = coIdn;
                    methodParam["logUsr"] = logUsr;
                    methodParam["status"] = "IS";
                    methodParam["exhrate"] = exhrate;
                    let insertResult = await execInsertTransactionMixQcM(methodParam, tpoolconn);
                    callback(null, insertResult);
                }
            }
        });
    } else if (processIdn == '') {
        outJson["result"] = resultFinal;
        outJson["status"] = "FAIL";
        outJson["message"] = "Please Verify processIdn Can not be blank!";
        callback(null, outJson);
    } else if (empIdn == '') {
        outJson["result"] = resultFinal;
        outJson["status"] = "FAIL";
        outJson["message"] = "Please Verify empIdn  Can not be blank!";
        callback(null, outJson);
    }
}

function execInsertTransactionMixQcM(methodParam, tpoolconn) {
    return new Promise(function (resolve, reject) {
        insertTransactionMixQcM(methodParam, tpoolconn, function (error, result) {
            if (error) {
                reject(error);
            }
            resolve(result);
        });
    });
}

function insertTransactionMixQcM(methodParam, tpoolconn, callback) {
    var processIdn = methodParam.processIdn || '';
    var empIdn = methodParam.empIdn || '';
    var status = methodParam.status;
    var optionDtl = methodParam.optionDtl || {};
    var coIdn = methodParam.coIdn;
    var logUsr = methodParam.logUsr;
    let exhrate = methodParam.exhrate;
    let addl_attr = {};
    addl_attr["exhrate"] = exhrate;
    let params = [];
    let fmt = {};
    let outJson = {};
    let resultFinal = {};

    if (processIdn != '' && empIdn != '') {
        let insertTransactionQ = "insert into transaction_mix_qc(process_idn,co_idn,co_serial,emp_idn,status,trns_ts,transaction_attr,approved_yn,created_ts,created_by,addl_attr) \n" +
            "values($1,$2,$3,$4,$5,current_timestamp,'" + JSON.stringify(optionDtl) + "',0,current_timestamp,$6,'" + JSON.stringify(addl_attr) + "') RETURNING transaction_mix_qc_idn";

        params.push(processIdn);
        params.push(coIdn);
        params.push("1");
        params.push(empIdn);
        params.push(status);
        params.push(logUsr);
        // console.log(insertTransactionQ);
        // console.log(params);
        coreDB.executeTransSql(tpoolconn, insertTransactionQ, params, fmt, function (error, result) {
            if (error) {
                coreDB.doTransRollBack(tpoolconn);
                outJson["status"] = "FAIL";
                outJson["message"] = "Error In insertTransactionM Method!" + error.message;
                callback(null, outJson);
            } else {
                //coreDB.doTransCommit(tpoolconn);
                var rowCount = result.rowCount;
                if (rowCount > 0) {
                    var resultRow = result.rows[0] || {};
                    outJson["transaction_mix_qc_idn"] = resultRow["transaction_mix_qc_idn"];
                    outJson["status"] = "SUCCESS";
                    outJson["message"] = "TransactionMixQC Inserted Sucessfully!";
                    callback(null, outJson);
                } else {
                    coreDB.doTransRollBack(tpoolconn);
                    outJson["status"] = "FAIL";
                    outJson["message"] = "TransactionMixQC Inserted Failed!";
                    callback(null, outJson);
                }
            }
        });
    } else if (processIdn == '') {
        outJson["result"] = resultFinal;
        outJson["status"] = "FAIL";
        outJson["message"] = "Please Verify processIdn Can not be blank!";
        callback(null, outJson);
    } else if (empIdn == '') {
        outJson["result"] = resultFinal;
        outJson["status"] = "FAIL";
        outJson["message"] = "Please Verify empIdn  Can not be blank!";
        callback(null, outJson);
    }
}

exports.insertTransactionMixQcD = function(methodParam, tpoolconn, callback) {
    var transaction_mix_qc_idn = methodParam.transaction_mix_qc_idn;
    var processType = methodParam.processType;
    var logUsr = methodParam.logUsr;
    let coIdn = methodParam.coIdn;

    var resultFinal = {};
    var outJson = {};
    let QCDStatus = "IS";
    //if (processType != 'IS')
    //    QCDStatus = "RT";
    let insertTransactionQ = "";
    var params = [];

    if(processType != 'IS'){
        insertTransactionQ = "insert into transaction_mix_d_qc(transaction_mix_qc_idn, stock_idn, qty, weight,stock_status,status, issval, rtnval,created_by,rtn_emp_idn,sync_stt,modified_ts,modified_by) " +
        "select $1, stock_idn, qty, weight,stock_status,$2,issval,rtnval,$3,empval,$4,current_timestamp,$5 from  temp_issuereturn_pkts where stt=1 ";

        params.push(transaction_mix_qc_idn);
        params.push(QCDStatus);
        params.push(logUsr);
        params.push("I");
        params.push(logUsr);

    } else {
        insertTransactionQ = "insert into transaction_mix_d_qc(transaction_mix_qc_idn, stock_idn, qty, weight,stock_status,status, issval, rtnval,created_by,rtn_emp_idn,sync_stt) " +
        "select $1, stock_idn, qty, weight,stock_status,$2,issval,rtnval,$3,empval,$4 from  temp_issuereturn_pkts where stt=1 ";

        params.push(transaction_mix_qc_idn);
        params.push(QCDStatus);
        params.push(logUsr);
        params.push("C");
    }
    

    //console.log(insertTransactionQ);
    // console.log(params);
    coreDB.executeTransSql(tpoolconn, insertTransactionQ, params, {}, async function (error, result) {
        if (error) {
            coreDB.doTransRollBack(tpoolconn);
            let redirectParam = {};
            redirectParam["logUsr"] = logUsr;
            redirectParam["coIdn"] = coIdn;
            redirectParam["errorMsg"] = error.message;
            redirectParam["transaction_mix_qc_idn"] = transaction_mix_qc_idn;
            let errorResult = await coreUtil.execUpdateErrorMix(redirectParam, tpoolconn);
            //console.log("d_qc error",error)
            outJson["result"] = resultFinal;
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In insertTransactionQcD Method!" + error.message;
            callback(null, outJson);
        } else {
            let rowCount = result.rowCount;
            if (rowCount > 0) {
                //coreDB.doTransCommit(tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "Transaction D Insertion Successfully!";
                callback(null, outJson);
            } else {
                coreDB.doTransRollBack(tpoolconn);
                let redirectParam = {};
                redirectParam["logUsr"] = logUsr;
                redirectParam["coIdn"] = coIdn;
                redirectParam["errorMsg"] = "Transaction D Inserted Failed!";
                redirectParam["transaction_mix_qc_idn"] = transaction_mix_qc_idn;
                let errorResult = await coreUtil.execUpdateErrorMix(redirectParam, tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "FAIL";
                outJson["message"] = "Transaction D Inserted Failed!";
                callback(null, outJson);
            }
        }
    });
}

exports.updateErrorMix = function(methodParam, tpoolconn, callback) {
    var logUsr = methodParam.logUsr;
    var coIdn = methodParam.coIdn || '';
    let errorMsg = methodParam.errorMsg || '';
    let transaction_mix_qc_idn = methodParam.transaction_mix_qc_idn || '';
    let statusUpdateYN = methodParam.statusUpdateYN || 'Y';

    let params = [];
    let outJson = {};
    let resultFinal = {};
    let conQ = '';
    if(statusUpdateYN == 'Y')
        conQ = " status='FL' , ";

    var updateQ = "update transaction_mix_d_qc d set "+conQ+" remarks = $1 " +
        ",modified_ts=current_timestamp,modified_by=$2 from temp_issuereturn_pkts tm " +
        "where d.stock_idn=tm.stock_idn  and tm.stt = 1  ";

    params = [];
    params.push(errorMsg);
    params.push(logUsr);

    if (transaction_mix_qc_idn != '') {
        updateQ += " and d.transaction_mix_qc_idn=$3 ";
        params.push(transaction_mix_qc_idn);
    }

    //console.log(updateQ)
    //console.log(params)
    coreDB.executeTransSql(tpoolconn, updateQ, params, {}, function (error, result) {
        if (error) {
            coreDB.doTransRollBack(tpoolconn);
            console.log(error.message)
            outJson["status"] = "FAIL";
            outJson["message"] = "Fail To Update transaction_mix_d_qc!" + error.message;
            outJson["result"] = resultFinal;
            callback(null, outJson);
        } else {
            //console.log("In stock_m");
            var updateStock = "update stock_m sm set status = d.stock_status  " +
                " ,modified_ts=current_timestamp,modified_by=$1 " +
                "from temp_issuereturn_pkts d where sm.stock_idn = d.stock_idn " +
                "and sm.co_idn=$2 " +
                "and sm.stt=1 and d.stt=1 ";

            params = [];
            params.push(logUsr);
            params.push(coIdn);

            if (transaction_mix_qc_idn != '') {
                updateStock += " and transaction_qc_idn=$3 ";
                params.push(transaction_mix_qc_idn);
            }

            //console.log(updateStock)
            //console.log(params)
            coreDB.executeTransSql(tpoolconn, updateStock, params, {}, function (error, result) {
                if (error) {
                    coreDB.doTransRollBack(tpoolconn);
                    outJson["status"] = "FAIL";
                    outJson["message"] = "Fail To Update Stock M!" + error.message;
                    outJson["result"] = resultFinal;
                    callback(null, outJson);
                } else {
                    //console.log("StockM Status Updated");

                    updateQ = "update temp_issuereturn_pkts d set flg='E' ,remarks = $1 " +
                        "where tm.stt = 1 ";

                    params = [];
                    params.push(errorMsg);
                    if (transaction_mix_qc_idn != '') {
                        updateQ += " and transaction_qc_idn=$2 ";
                        params.push(transaction_mix_qc_idn);
                    }

                    //console.log(updateQ)
                    //console.log(params)
                    coreDB.executeTransSql(tpoolconn, updateQ, params, {}, function (error, result) {
                        if (error) {
                            coreDB.doTransRollBack(tpoolconn);
                            outJson["status"] = "FAIL";
                            outJson["message"] = "Fail To Update temp_issuereturn_pkts!" + error.message;
                            outJson["result"] = resultFinal;
                            callback(null, outJson);
                        } else {
                            //coreDB.doTransCommit(tpoolconn);

                            outJson["status"] = "SUCCESS";
                            outJson["message"] = "SUCCESS";
                            outJson["result"] = resultFinal;
                            callback(null, outJson);
                        }
                    })
                }
            })
        }
    })
}

exports.postAssortIssueProcedureMix = function(methodParam, tpoolconn, callback) {
    var transaction_mix_qc_idn = methodParam.transaction_mix_qc_idn;
    let coIdn = methodParam.coIdn;
    let logUsr = methodParam.logUsr;
    var resultFinal = {};
    var outJson = {};

    let sql = "select post_assort_issue_mix($1, $2) ";

    var params = [];
    params.push(coIdn);
    params.push(transaction_mix_qc_idn);

    //console.log(sql);
    //console.log(params);
    coreDB.executeTransSql(tpoolconn, sql, params, {}, async function (error, result) {
        if (error) {
            coreDB.doTransRollBack(tpoolconn);
            let redirectParam = {};
            redirectParam["logUsr"] = logUsr;
            redirectParam["coIdn"] = coIdn;
            redirectParam["errorMsg"] = error.message;
            redirectParam["transaction_mix_qc_idn"] = transaction_mix_qc_idn;
            let errorResult = await coreUtil.execUpdateErrorMix(redirectParam, tpoolconn);
            console.log(error.message)
            outJson["result"] = resultFinal;
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In post_assort_issue_mix Method!" + error.message;
            callback(null, outJson);
        } else {
            let rowCount = result.rowCount;
            if (rowCount > 0) {
                //coreDB.doTransCommit(tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "post_assort_issue_mix done Successfully!";
                callback(null, outJson);
            } else {
                coreDB.doTransRollBack(tpoolconn);
                let redirectParam = {};
                redirectParam["logUsr"] = logUsr;
                redirectParam["coIdn"] = coIdn;
                redirectParam["errorMsg"] = "post_assort_issue_mix procedure Failed!";
                redirectParam["transaction_mix_qc_idn"] = transaction_mix_qc_idn;
                let errorResult = await coreUtil.execUpdateErrorMix(redirectParam, tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "FAIL";
                outJson["message"] = "post_assort_issue_mix procedure Failed!";
                callback(null, outJson);
            }
        }
    });
}

exports.preAssortReturnProcedureMix = function(methodParam, tpoolconn, callback) {
    var transaction_mix_qc_idn = methodParam.transaction_mix_qc_idn;
    let coIdn = methodParam.coIdn;
    let logUsr = methodParam.logUsr;
    var resultFinal = {};
    var outJson = {};

    let sql = "select pre_assort_return_mix($1, $2) ";


    var params = [];
    params.push(coIdn);
    params.push(transaction_mix_qc_idn);

    //console.log(sql);
    //console.log(params);
    coreDB.executeTransSql(tpoolconn, sql, params, {}, async function (error, result) {
        if (error) {
            coreDB.doTransRollBack(tpoolconn);
            let redirectParam = {};
            redirectParam["logUsr"] = logUsr;
            redirectParam["coIdn"] = coIdn;
            redirectParam["errorMsg"] = error.message;
            redirectParam["transaction_mix_qc_idn"] = transaction_mix_qc_idn;
            let errorResult = await coreUtil.execUpdateErrorMix(redirectParam, tpoolconn);
            console.log("pre_assort_return_mix",error.message)
            outJson["result"] = resultFinal;
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In pre_assort_return_mix Method!" + error.message;
            callback(null, outJson);
        } else {
            let rowCount = result.rowCount;
            if (rowCount > 0) {
                //coreDB.doTransCommit(tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "pre_assort_return_mix done Successfully!";
                callback(null, outJson);
            } else {
                coreDB.doTransRollBack(tpoolconn);
                let redirectParam = {};
                redirectParam["logUsr"] = logUsr;
                redirectParam["coIdn"] = coIdn;
                redirectParam["errorMsg"] = "pre_assort_return_mix procedure Failed!";
                redirectParam["transaction_mix_qc_idn"] = transaction_mix_qc_idn;
                let errorResult = await coreUtil.execUpdateErrorMix(redirectParam, tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "FAIL";
                outJson["message"] = "pre_assort_return_mix procedure Failed!";
                callback(null, outJson);
            }
        }
    });
}

exports.getTransactionMixSummary = function(methodParam, tpoolconn, callback) {
    let transaction_mix_qc_idn = methodParam.transaction_mix_qc_idn;
    let coIdn = methodParam.coIdn;
    let summaryTrnsDtl = {};
    let outJson = {};

    let sql = "select tq.transaction_mix_qc_idn,tq.co_serial,sp.print,sp.nme, sp.grp,sp.process_idn, "+
            "get_nme(tq.emp_idn) employee,tq.emp_idn, "+
            "sum(tdq.qty) qty,sum(tdq.weight) crtwt "+
            "from transaction_mix_qc tq,transaction_mix_d_qc tdq, "+
            "stock_m sm,stock_process sp "+
            "where tq.transaction_mix_qc_idn = tdq.transaction_mix_qc_idn "+
            "and tq.process_idn= sp.process_idn "+
            "and tdq.stock_idn=sm.stock_idn "+
            "and sm.co_idn=$1 and sp.co_idn=$2 and tq.co_idn = sp.co_idn "+
            "and sp.stt=1 and tdq.status='IS' "+
            "and sp.iss_status=sm.status "+
            "and tq.transaction_mix_qc_idn = $3 "+
            "group by tq.transaction_mix_qc_idn,sp.print,tq.emp_idn, sp.grp,sp.nme,sp.process_idn,tq.co_serial ";


    let params = [];
    params.push(coIdn);
    params.push(coIdn);
    params.push(transaction_mix_qc_idn);

    //console.log(sql);
    // console.log(params);
    coreDB.executeTransSql(tpoolconn, sql, params, {}, async function (error, result) {
        if (error) {
            outJson["result"] = summaryTrnsDtl;
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In getTransactionSummary Method!" + error.message;
            callback(null, outJson);
        } else {
            let rowCount = result.rowCount;
            if (rowCount > 0) {
                summaryTrnsDtl["transactionIdn"] = result.rows[0].transaction_mix_qc_idn;
                summaryTrnsDtl["co_serial"] = result.rows[0].co_serial;
                summaryTrnsDtl["processNme"] = result.rows[0].nme;
                summaryTrnsDtl["qty"] = result.rows[0].qty;
                summaryTrnsDtl["weight"] = result.rows[0].crtwt;
                summaryTrnsDtl["employee"] = result.rows[0].employee;
                summaryTrnsDtl["emp_idn"] = result.rows[0].emp_idn;
                summaryTrnsDtl["processGrp"] = result.rows[0].grp;
                summaryTrnsDtl["processPrint"] = result.rows[0].print;
                summaryTrnsDtl["processIdn"] = result.rows[0].process_idn;

                outJson["result"] = summaryTrnsDtl;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "SUCCESS";
                callback(null, outJson);
            } else {
                outJson["result"] = summaryTrnsDtl;
                outJson["status"] = "FAIL";
                outJson["message"] = "Sorry no result found";
                callback(null, outJson);
            }
        }
    })
}

exports.postAssortReturnProcMix = function(redirectParam, tpoolconn, callback) {
    var transaction_mix_qc_idn = redirectParam.transaction_mix_qc_idn;
    let coIdn = redirectParam.coIdn;
    let stock_idn = redirectParam.stock_idn;
    let processIdn = redirectParam.processIdn;
    let logUsr = redirectParam.logUsr;
    var resultFinal = {};
    var outJson = {};

    let sql = "select post_assort_return_mix($1, $2, $3, $4)";


    var params = [];
    params.push(coIdn);
    params.push(processIdn);
    params.push(transaction_mix_qc_idn);
    params.push(stock_idn);

    //console.log(sql);
    //console.log(params);
    coreDB.executeTransSql(tpoolconn, sql, params, {}, async function (error, result) {
        if (error) {
            coreDB.doTransRollBack(tpoolconn);
            let redirectParam = {};
            redirectParam["logUsr"] = logUsr;
            redirectParam["coIdn"] = coIdn;
            redirectParam["errorMsg"] = error.message;
            redirectParam["transaction_mix_qc_idn"] = transaction_mix_qc_idn;
            redirectParam["statusUpdateYN"] = "N";
            let errorResult = await coreUtil.execUpdateErrorMix(redirectParam, tpoolconn);
            console.log(error.message)
            outJson["result"] = resultFinal;
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In post_assort_return_mix Method!" + error.message;
            callback(null, outJson);
        } else {
            let rowCount = result.rowCount;
            if (rowCount > 0) {
                //coreDB.doTransCommit(tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "SUCCESS";
                outJson["message"] = "post_assort_return_mix done Successfully!";
                callback(null, outJson);
            } else {
                coreDB.doTransRollBack(tpoolconn);
                let redirectParam = {};
                redirectParam["logUsr"] = logUsr;
                redirectParam["coIdn"] = coIdn;
                redirectParam["errorMsg"] = "post_assort_return_mix procedure Failed!";
                redirectParam["transaction_mix_qc_idn"] = transaction_mix_qc_idn;
                redirectParam["statusUpdateYN"] = "N";
                let errorResult = await coreUtil.execUpdateErrorMix(redirectParam, tpoolconn);
                outJson["result"] = resultFinal;
                outJson["status"] = "FAIL";
                outJson["message"] = "post_assort_return_mix procedure Failed!";
                callback(null, outJson);
            }
        }
    });
}

exports.getSaveRepairDataToMFG = function(methodParam, tpoolconn, callback) {
    let transaction_qc_idn = methodParam.transaction_qc_idn;
    let coIdn = methodParam.coIdn;
    let username = methodParam.username;
    let processName = methodParam.processName || '';
    let prefix = methodParam.prefix || '';
    let trnsPktList = [];
    let outJson = {};
    let sql = '';
    var cachedUrl = require('qaq-core-util').cachedUrl;

    if(processName == 'repairing_issue'){
        sql = "select tq.transaction_qc_idn memoIdn,tq.co_serial,sm.attr ->> 'lot' lot  \n"+
            ",sm.attr ->> 'lot_map' lot_map ,sm.attr ->> 'vnm' vnm,to_char(tq.trns_ts,'yyyy-MM-dd') trns_ts, \n"+
            "tdq.qty qty,tdq.weight crtwt,coalesce(sm.attr ->> 'rough_weight',sm.attr ->> 'rough_cts') rough_weight, \n"+
            "trunc(cast(coalesce(sm.attr ->> 'mfg_pri',sm.attr ->> 'rte') as numeric)*tdq.weight,2) amt,  \n"+
            "sm.attr->>'pur' pur,sm.stock_type \n"+
            ",sm.attr->>'rep_cmt' rep_cmt,tq.addl_attr ->> 'exhrate' exhrate \n"+
            "from transaction_qc tq,transaction_d_qc tdq,  \n"+
            "stock_m sm \n"+
            "where tq.transaction_qc_idn = tdq.transaction_qc_idn \n"+
            "and tdq.stock_idn=sm.stock_idn  \n"+
            "and sm.co_idn=$1  \n"+
            "and tdq.status='IS'  \n"+
            "and tq.transaction_qc_idn = $2  ";
    } else if(processName == 'mix_repairing'){
        sql = "select tq.transaction_mix_qc_idn memoIdn,tq.co_serial,sm.attr ->> 'lot' lot  \n"+
            ",sm.attr ->> 'lot_map' lot_map ,sm.attr ->> 'vnm' vnm,to_char(tq.trns_ts,'yyyy-MM-dd') trns_ts, \n"+
            "tdq.qty qty,tdq.weight crtwt,coalesce(sm.attr ->> 'rough_weight',sm.attr ->> 'rough_cts') rough_weight, \n"+
            "trunc(cast(coalesce(sm.attr ->> 'cp',sm.attr ->> 'mfg_pri') as numeric)*tdq.weight,2) amt,  \n"+
            "sm.attr->>'pur' pur,sm.stock_type \n"+
            ",sm.attr->>'rep_cmt' rep_cmt,tq.addl_attr ->> 'exhrate' exhrate \n"+
            "from transaction_mix_qc tq,transaction_mix_d_qc tdq,  \n"+
            "stock_m sm \n"+
            "where tq.transaction_mix_qc_idn = tdq.transaction_mix_qc_idn  \n"+
            "and tdq.stock_idn=sm.stock_idn \n"+
            "and sm.co_idn=$1 \n"+
            "and tdq.status='IS'  \n"+
            "and tq.transaction_mix_qc_idn = $2  ";
    }

    let params = [];
    params.push(coIdn);
    params.push(transaction_qc_idn);

    //console.log(sql);
    //console.log(params);
    coreDB.executeTransSql(tpoolconn, sql, params, {}, async function (error, result) {
        if (error) {
            outJson["status"] = "FAIL";
            outJson["message"] = "Error In getSaveRepairDataToMFG Method!" + error.message;
            callback(null, outJson);
        } else {
            let len = result.rows.length;
            //console.log("repairingLen"+len);
            if (len > 0) {
                let productAttributeDtl = await coreUtil.getCache(prefix+"productAttributeDtl_"+coIdn,cachedUrl);
                if(productAttributeDtl == null){
                    outJson["status"]="FAIL";
                    outJson["message"]="Fail to get Product Sub Attribute";
                    callback(null,outJson);
                }else{
                    productAttributeDtl = JSON.parse(productAttributeDtl);
                    let attr = "pur";
                    let prpSort=productAttributeDtl[attr+"#S"] || [];
                    let prpDisplay=productAttributeDtl[attr+"#P"] || [];

                    for(let i=0;i<len;i++){
                        let map = {};
                        let resultRows = result.rows[i] || {};
                        map["rcvd_sht_nmbr"] = resultRows.memoidn ;
                        let vnm =  resultRows.vnm;
                        map["rcvd_lt_nmbr"] = resultRows.lot || vnm;
                        map["rcvd_pckt_nmbr"] = resultRows.lot_map || vnm;
                        map["rcvd_pckt_id"] = vnm;
                        map["snt_dt"] = resultRows.trns_ts;
                        map["pcs"] = resultRows.qty;
                        map["cnts"] = resultRows.crtwt;
                        map["rgh_cnt"] = resultRows.rough_weight || '0';
                        let exhrate = resultRows.exhrate || '1';
                        let amt = resultRows.amt || '0';
                        let rsamt = parseFloat(amt) * parseFloat(exhrate);
                        map["vlu_fe"] = coreUtil.floorFigure(rsamt,2);
                        let stock_type = resultRows.stock_type;
                        let pur = resultRows.pur || '';
                        if(pur != '')
                            pur=prpDisplay[prpSort.indexOf(parseInt(pur))] || '';
                        let pckt_typ = "EXISTING";
                        if(stock_type == 'MIX')
                            pckt_typ = "BULK";
                        if(pur != 'NA')
                            pckt_typ = "OUTSIDE";   

                        map["rcvd_pckt_typ"] = pckt_typ;
                        map["rcvd_rep_rmk"] = resultRows.rep_cmt || '';
                        map["rcvd_rep_typ"] = "";
                        trnsPktList.push(map);
                    }
                
                    let methodParamLocal = {};
                    methodParamLocal["coIdn"] = coIdn;
                    methodParamLocal["trnsPktList"] = trnsPktList;
                    methodParamLocal["username"] = username;
                    let mfgResult = await execSaveMFGDetails(methodParamLocal);
                    let data = {};
                    data["repairMFGData"] = mfgResult.message || '';

                    let updatetransale = "";
                    if(processName == 'repairing_issue'){
                        updatetransale = "update transaction_qc  set  transaction_attr= transaction_attr ||'" + JSON.stringify(data) + "' \n" +
                        ",modified_ts= current_timestamp,modified_by = $1 \n"+
                        " where transaction_qc_idn = $2 ";
                    } else if(processName == 'mix_repairing'){
                        updatetransale = "update transaction_mix_qc  set  transaction_attr= transaction_attr ||'" + JSON.stringify(data) + "' \n" +
                        ",modified_ts= current_timestamp,modified_by = $1 \n"+
                        " where transaction_mix_qc_idn = $2 ";
                    }
                    params = [];
                    params.push(username);
                    params.push(transaction_qc_idn);

                    //console.log(sql);
                    //console.log(params);
                    coreDB.executeTransSql(tpoolconn, updatetransale, params, {}, async function (error, result) {
                        if (error) {
                            outJson["status"] = "FAIL";
                            outJson["message"] = "Error In transactionQcUpdation Method!" + error.message;
                            callback(null, outJson);
                        } else {               
                            outJson["status"] = "SUCCESS";
                            outJson["message"] = "SUCCESS";
                            callback(null, outJson);
                        }
                    })
                }
            } else {
                outJson["status"] = "FAIL";
                outJson["message"] = "Sorry no result found";
                callback(null, outJson);
            }
        }
    })
}

function execSaveMFGDetails(methodParam) {
    return new Promise(function (resolve, reject) {
        saveMFGDetails( methodParam,  function (error, result) {
            if (error) {
                reject(error);
            }
            resolve(result);
        });
    });
}

async function saveMFGDetails(paramJson,callback) {
    var coIdn = paramJson.coIdn;
    let mfgPoolName = paramJson.mfgPoolName || "MFGPOOL";
    let username = paramJson.username;
    let trnsPktList = paramJson.trnsPktList || [];
    let outJson = {};
    let pktMap = {};
    pktMap["packets"] = trnsPktList;
    //let query =  JSON.stringify(pktMap);
    //console.log("pktMap",  query );

    var poolsList= require('qaq-core-db').poolsList;
    mfgPoolName = mfgPoolName.trim();
    var pool = poolsList[mfgPoolName] || '';
    //console.log("pool",pool);
    if(pool !=''){
        coreDB.getTransPoolConnect(pool,function(error,mfgconnection){
            if(error){  
                console.log(error);
                outJson["status"]="FAIL";
                outJson["message"]="Fail To Get MFG Conection!";
                callback(null,outJson);   
            }else{

                let fmt = {};
                let params=[];
                
                var sql=`SELECT * from public.usp_set_plsrep_rcvd(p_usr => substr($1, 0, 10)::varchar, p_trmn =>  substr($2, 0, 10)::varchar, 
                        p_json=>'${JSON.stringify(pktMap)}'::json) `;

                params.push(username);
                params.push(username);
                //console.log(sql)
                //console.log(params)
                coreDB.executeTransSql(mfgconnection,sql,params,fmt,function(error,result){
                    if(error){
                        console.log(error);
                        coreDB.doRelease(mfgconnection);
                        outJson["status"]="FAIL";
                        outJson["message"]="Error In saveMFGDetails Method!";
                        callback(null,outJson);
                    }else{
                        var rowCount=result.rowCount;
                        //console.log("result",result);
                        if(rowCount>0){
                            coreDB.doRelease(mfgconnection);
                            outJson["status"]="SUCCESS";
                            outJson["message"]=result.rows;
                            callback(null,outJson);
                        }else{
                            coreDB.doRelease(mfgconnection);
                            outJson["status"]="FAIL";
                            outJson["message"]="Repairing Data not updated in MFG DB";
                            callback(null,outJson);
                        }
                    }
                })
            }
        })
    } else{
        outJson["result"]='';
        outJson["status"]="FAIL";
        outJson["message"]="Please Verify Pool from PoolList can not be blank!";
        callback(null,outJson);
    }
}
